#pragma once

#ifndef __PHONEPOSITION_H__
#define __PHONEPOSITION_H__

#include "Common.h"
#include <gpsapi.h>
#include <cmath>
#include "ril.h"

/**
 *	<<singleton>>
 *	GPS
 *		Retrieve Phone position via RIL/GPS
 **/
class GPS
{
	public:
		static GPS *self(DWORD dwTimeout, DWORD dwMaximumAge);

		BOOL	GpsReady();
		BOOL	RadioReady();

		BOOL	getGPS(GPS_POSITION *);
		BOOL	getCELL(RILCELLTOWERINFO *);

		void	Stop();
		BOOL	Start();

		DOUBLE  VincentFormula(DOUBLE lat1, DOUBLE lon1, DOUBLE lat2, DOUBLE lon2);

	protected:
		GPS();	// .ctor

		static GPS *_instance;	// Global self
		
		/**
		* Inizializza l'oggetto, torna FALSE se l'inizializzazione fallisce.
		* dwTimeout e dwMaximumAge sono in millisecondi e rappresentano rispettivamente
		* il tempo massimo di attesa per ottenere una posizione valida e l'eta massima
		* della coordinata richiesta.
		**/
		BOOL	Initialize(DWORD dwTimeout, DWORD dwMaximumAge);

		BOOL	RefreshData(short mask);	// Try to refresh data

	private: ~GPS();	// .dtor

	private:
		DWORD	_dwTimeout;			// constructor parameters
		DWORD	_dwMaximumAge;
		DWORD	_dwTickCount;		// old request tick count

		HANDLE	_hMutex;			// mutex
		INT		_iReference;		// Reference counter

		HANDLE	_hGpsDevice;		// device handle
		HRIL	_hRil;

		GPS_POSITION	_gpsPosition;
		RILCELLTOWERINFO _rilCellTowerInfo;
		
		DWORD	_dwLastGps;			// time of last GPS
		DWORD	_dwLastCell;		// time of last CELL
		
		HRESULT _cellRequestID;		// cellRequestID
		HANDLE	_hCellEvent;		// hCellEvent

		static	BOOL _bRadioReady;	// Radio is READY
		static	BOOL _bGpsReady;	// GPS Ready
		static volatile LONG lLock;	// Il nostro mutex

	private:
		static void _RIL_Callback(DWORD dwCode, HRESULT hrCmdID, const void* lpData, DWORD cbData, DWORD dwParam);
		static void _RIL_Notify(DWORD dwCode, const void* lpData, DWORD cbData, DWORD dwParam);

		void RIL_Callback(DWORD dwCode, HRESULT hrCmdID, const void* lpData, DWORD cbData);
		void RIL_Notify(DWORD dwCode, const void* lpData, DWORD cbData);
};

#endif
