#pragma once

#include <map>

#include "Conf.h"
#include "Log.h"
#include "MAPICommon.h"
#include "MAPI.h"
#include "MAPIStorageAdviser.h"
#include "MAPIAgentConfiguration.h"
#include "MAPIAgentClassFilter.h"

class MAPIMessage;
class MAPIStorage;  

class MAPIAgent // Singleton
{
private:
  static MAPIAgent* _instance;
  static int _refCount;
  static volatile LONG _lLock;

  MAPIStorageAdviser adviser;
  
  HRESULT FolderPath( tree<MAPIFolder*>::iterator folder_iter, 
                    MAPIStorage* storage, 
                    std::map<int, LPTSTR> &folder_names, 
                    LPTSTR lpPath );

  MAPIAgentConfiguration* _conf;

  MAPI* _mapi;

  DWORD _dwCollectorThreadID;
  HANDLE _hCollectorThread;

  bool _HasToQuit;

protected:
  MAPIAgent(void);

  BOOL AdviseStore(MAPIStorage* pStorage)
  {
    _ASSERT(pStorage);

    if (!pStorage)
      return FALSE;

    adviser.AddSink(pStorage->GetIMsgStorePtr());
    return TRUE;
  }
  
public:
  
  static MAPIAgent* Instance();
  static void Release();
  static void Destroy();
  
  LPWSTR ConfigurationTag() { return _conf->Tag(); }
  
  void Quit() { _HasToQuit = true; }
  bool IsQuitting() { return _HasToQuit; }
  
  BOOL Init(LPBYTE lpConfig, DWORD cbConfig);
  HRESULT Run(DWORD nRun);
  HRESULT AdviseStores();
  HRESULT Collect(DWORD nRun);
   BOOL MAPIAgent::Accept( MAPIMessage* msg, FilterTypes FilterType );
  
  void WaitForCollector()
  {
    if (_conf->HasCollectConfig())
    {
      DWORD ret = WaitForSingleObject(_hCollectorThread, INFINITE);
      if (ret == WAIT_OBJECT_0)
        return;

    }
  }
  
  BOOL setConfiguration(LPBYTE lpConfig, DWORD cbConfig) 
  {
    ASSERT(lpConfig);
    ASSERT(cbConfig > 0);
    
    if (_conf)
    {
      delete _conf;
      _conf = NULL;
    }
    
    _conf = MAPIAgentConfiguration::Unserialize(lpConfig, cbConfig);
    
    /*
    Log* pLog = new Log();
    // pLog->WriteMarkup(AGENT_EMAIL, pData, cbLen);
    delete pLog;
    */

    return TRUE;
  }
  
  MAPIAgentClassFilter* IsMessageClassAcceptable(LPTSTR ClassName, FilterTypes FilterType) 
  {
    _ASSERT(ClassName);
    return _conf->MatchClassName(ClassName, FilterType);
  }
  
  virtual ~MAPIAgent(void)
  {
    if (_mapi)
      delete _mapi;

    if (_conf)
      delete _conf;
  }
};
