#include <list>

#include "MAPICommon.h"
#include "MAPIAgent.h"
#include "MAPIMessage.h"
#include "MAPIAttachment.h"
#include "MAPISerialize.h"

#define INITGUID
#define USES_IID_IMAPITable
#include <mapiguid.h>

MAPIMessage::MAPIMessage(IMessage *pMsg)
: _pMsg(pMsg), _pMsgProps(NULL), _pAttachTable(NULL), _cbMsgProps(0), _lpMimeBody(NULL), _cbMimeBodySize(0),
_Class(NULL), _From(NULL), _Subject(NULL), _DeliveryTime(NULL), _hasAttach(FALSE)
{ 
  LPTSTR alphabet = L"abcdefABCDEF1234567890";

  ZeroMemory(&_header, sizeof(_header));
  
  for (UINT i = 0; i < _filenameLength; i++)
    _lpFilename[i] = alphabet[rand() % _tcslen(alphabet)];
}

MAPIMessage::~MAPIMessage(void)
{ 
  if (_lpMimeBody) LocalFree(_lpMimeBody);
  // if (_To) LocalFree(_To);
  if (_pMsgProps) MAPIFreeBuffer(_pMsgProps);
  if (_pAttachTable) _pAttachTable->Release();
  if (_pMsg) _pMsg->Release();

  /*
  for (std::list<MAPIAttachment*>::iterator i = attachments.begin(); i < attachments.end(); i++)
    delete *i;
  attachments.clear();
  */
}

HRESULT MAPIMessage::Parse(void)
{
  HRESULT hr = E_FAIL;
  
  hr = _GetProperties();
  if (FAILED(hr))
  {
    return E_FAIL;
  }
  
  hr = _ParseProperties();
  if (FAILED(hr))
  {
    return E_FAIL;
  }
  
  BOOL accepted = MAPIAgent::Instance()->Accept(this, COLLECT);
  if (accepted == FALSE)
    return E_FAIL;
  
  
  hr = _GetMessageBody();
  if (FAILED(hr))
  {
  }
 
  
	return S_OK;
}

HRESULT MAPIMessage::_GetProperties( void )
{
  ULONG rgTags[] = {5,  PR_MESSAGE_CLASS,
                        PR_MESSAGE_DELIVERY_TIME,
                        PR_SENDER_EMAIL_ADDRESS,
                        PR_SUBJECT,
                        PR_HASATTACH};
  
  HRESULT hr = _pMsg->GetProps((LPSPropTagArray)rgTags, MAPI_UNICODE, &_cbMsgProps, &_pMsgProps);
  if (hr != S_OK)
    return E_FAIL;
  
  return S_OK;
}

HRESULT MAPIMessage::_ParseProperties( void )
{
  for (ULONG idx = 0; idx < _cbMsgProps; idx++)
  {
    switch(_pMsgProps[idx].ulPropTag)
    {
    case PR_MESSAGE_CLASS:
      _Class = _pMsgProps[idx].Value.lpszW;
      break;
    case PR_MESSAGE_DELIVERY_TIME:
      _DeliveryTime = &_pMsgProps[idx].Value.ft;
      CopyMemory(&_header.DeliveryTime, _DeliveryTime, sizeof(FILETIME));
      break;
    case PR_SENDER_EMAIL_ADDRESS:
      _From = _pMsgProps[idx].Value.lpszW;
      break;
    case PR_SUBJECT:
      _Subject = _pMsgProps[idx].Value.lpszW;
      break;
    case PR_HASATTACH:
      _hasAttach = static_cast<BOOL>(_pMsgProps[idx].Value.i);
      break;
    /*
    case PR_MESSAGE_FLAGS:
      _header.Flags = _pMsgProps[idx].Value.l;
      break;
    case PR_MSG_STATUS:
      _header.Status = _pMsgProps[idx].Value.l;
      break;
    case PR_LAST_MODIFICATION_TIME:
      break;
    case PR_MESSAGE_SIZE:
      _header.Size = _pMsgProps[idx].Value.l;
      break;
    case PR_BODY:
      wprintf(_T("PR_BODY\n"));
      break;
    case PR_BODY_HTML:
      wprintf(_T("PR_BODY_HTML\n"));
      break;
    */
    default:
      {
        /*
        LONG type = PROP_TYPE(_pMsgProps[idx].ulPropTag);
        LONG id = PROP_ID(_pMsgProps[idx].ulPropTag);
        wprintf(_T("Unknown tag [type %ld id %08x]\n"), type, id);
        if (type == PT_UNICODE)
          wprintf(_T("\t\tString: %s\n"), _pMsgProps[idx].Value.lpszW);
        else if (type == PT_STRING8)
          wprintf(_T("\t\tString: %s\n"), _pMsgProps[idx].Value.lpszA);
        */
      }
      break;
    }

    Sleep(5);
  }

  return S_OK;
}

/*
HRESULT MAPIMessage::_GetRecipients( void )
{
  if (_To != NULL) LocalFree(_To);
  _To = (LPTSTR) LocalAlloc(LPTR, MAXBUF * sizeof(TCHAR));
  
  HRESULT hr = _WriteRecipientsToString(_To);
  if (FAILED(hr))
  {
    LocalFree(_To);
    _To = NULL;
    return E_FAIL;
  }
  
  return S_OK;
}
*/

HRESULT MAPIMessage::_WriteRecipientsToString( LPTSTR pszTo )
{
  _ASSERT(_pMsg);

  HRESULT hr = E_FAIL;
  BOOL fFirstRecipient =   TRUE;
  size_t pcchA, pcchB;
  
  LPMAPITABLE pRecipientTable = NULL;
  hr = _pMsg->GetRecipientTable (MAPI_UNICODE, &pRecipientTable);
  if (FAILED(hr)) {
	  return E_FAIL;
  }
  
  for (;;)
  {
    SRowSet * pRowSet = NULL;
    
    // Copy properties to the ADRLIST
    hr = pRecipientTable->QueryRows (1, 0, &pRowSet);
    if (FAILED(hr))
    {
      pRecipientTable->Release();
      return E_FAIL;
    }
    
    // Did we hit the end of the table?
    if (pRowSet->cRows != 1)
      break; 
    
    // Point just past the last property
    LPSPropValue pspvLast = pRowSet->aRow[0].lpProps + pRowSet->aRow[0].cValues;
    
    // Loop through all the properties returned for this row
    for (LPSPropValue pPropValue = pRowSet->aRow[0].lpProps; pPropValue < pspvLast; ++pPropValue)
    {
      switch (pPropValue->ulPropTag)
      {
        //At this point you may also be interested on PR_DISPLAY_NAME
      case PR_DISPLAY_NAME:
      case PR_EMAIL_ADDRESS:
      case PR_EMAIL_ADDRESS_A:
        if (!fFirstRecipient) 
        {
          hr = StringCchLength(pszTo, MAXBUF, &pcchA);
          if (FAILED(hr)) 
          {
            FreeProws(pRowSet);
            pRecipientTable->Release();           
            return E_FAIL;
          }

          hr = StringCchCat(pszTo, pcchA+2+1, TEXT(", "));
          if (FAILED(hr)) 
          {
            FreeProws(pRowSet);
            pRecipientTable->Release();
            return E_FAIL;
          }
        }
        else 
        {
          fFirstRecipient = FALSE;
        }
        
        hr = StringCchLength(pszTo, MAXBUF, &pcchA);
        if (FAILED(hr)) 
        {
          FreeProws(pRowSet);
          pRecipientTable->Release();
          return E_FAIL;
        }
        
        hr = StringCchLength(pPropValue->Value.lpszW, MAXBUF, &pcchB);
        if (FAILED(hr)) 
        {
          FreeProws(pRowSet);
          pRecipientTable->Release();
          return E_FAIL;
        }
        
        hr = StringCchCat(pszTo, pcchA+pcchB+1, pPropValue->Value.lpszW);
        if (FAILED(hr)) 
        {
          FreeProws(pRowSet);
          pRecipientTable->Release();
          return E_FAIL;
        }
        
        break;
        
      default:
        break;
      }
    }
    
    FreeProws (pRowSet);
    pRowSet = NULL;
  }
  
  pRecipientTable->Release();
  
  return S_OK;
}

HRESULT MAPIMessage::_FormatDateTime( LPFILETIME ft, LPTSTR lpwDate, LPTSTR lpwTime )
{
  _ASSERT(ft);
  _ASSERT(lpwDate);
  _ASSERT(lpwTime);
  
  SYSTEMTIME st = {0};
  FileTimeToSystemTime(ft, &st);
  
  int lenDate = GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, NULL, 0);
  int err = GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, lpwDate, lenDate);
  if (err == 0)
    return E_FAIL;
  
  int lenTime = GetTimeFormat(LOCALE_USER_DEFAULT, TIME_FORCE24HOURFORMAT | TIME_NOTIMEMARKER, &st, NULL, NULL, 0);
  err = GetTimeFormat(LOCALE_USER_DEFAULT, 
    TIME_FORCE24HOURFORMAT | TIME_NOTIMEMARKER, 
    &st, 
    NULL, 
    lpwTime, 
    lenTime);
  if (err == 0)
    return E_FAIL;
  
  return S_OK;
}

HRESULT MAPIMessage::_GetMessageBody()
{
  HRESULT hr = E_FAIL;
  // TCHAR dbgString[255] = {0};
  
  LPMAPIPROP pProp = NULL; 
  hr = _pMsg->QueryInterface(IID_IMAPIProp, (LPVOID *) &pProp);
  if (FAILED(hr))
    return E_FAIL;
  
  ULONG cValues = 0;
  LPSPropValue rgMsgProps= NULL;
  ULONG rgTags[] = {2, PR_MSG_STATUS, PR_MESSAGE_FLAGS};
  hr = pProp->GetProps((LPSPropTagArray)rgTags, MAPI_UNICODE, &cValues, &rgMsgProps);
  if (FAILED(hr)) 
    return E_FAIL;
  
  pProp->Release();
  
  // wsprintf(dbgString, _T("Props %08x\r\n"), rgMsgProps[0].Value.ul);
  // OutputDebugString(dbgString);
  
  BOOL bIsMime = (
    (rgMsgProps[0].ulPropTag == PR_MSG_STATUS) && 
    (    
    (rgMsgProps[0].Value.ul & MSGSTATUS_HAS_PR_CE_MIME_TEXT) ||
    (rgMsgProps[0].Value.ul & MSGSTATUS_HAS_PR_CE_CRYPT_MIME_TEXT) ||
    (rgMsgProps[0].Value.ul & MSGSTATUS_HAS_PR_CE_SMIME_TEXT)
    )
    )
    && 
    !(rgMsgProps[0].Value.ul & MSGSTATUS_HAS_PR_BODY);
  
  MAPIFreeBuffer(rgMsgProps);
  
  // wsprintf(dbgString, _T("MIME %s\r\n"), bIsMime == TRUE ? _T("true") : _T("false"));
  // OutputDebugString(dbgString);
  
  if (bIsMime == TRUE)
  {
    IStream* istream = NULL;
    hr = _pMsg->OpenProperty(PR_CE_MIME_TEXT, &IID_IStream, STGM_READ, 0, (LPUNKNOWN*)&istream);
    if (FAILED(hr))
    {
      return E_FAIL;
    }

    STATSTG stg;
    hr = istream->Stat(&stg, STATFLAG_NONAME);
    if (FAILED(hr))
    {
      istream->Release();
      return E_FAIL;
    }

    _cbMimeBodySize = stg.cbSize.LowPart;
    _lpMimeBody = (LPBYTE)LocalAlloc(LPTR, _cbMimeBodySize);
    
    // wsprintf(dbgString, _T("Body size: %d\r\n"), cbBody);
    // OutputDebugString(dbgString);
    
    ULONG cbRead = 0;
    hr = istream->Read(_lpMimeBody, _cbMimeBodySize, &cbRead);
    if (FAILED(hr))
    {
      LocalFree(_lpMimeBody);
      istream->Release();
      return E_FAIL;
    }
        
    istream->Release();
  }

  return S_OK;
}

HRESULT MAPIMessage::_OpenAttachments()
{
  HRESULT hr = E_FAIL;
  
  hr = _pMsg->GetAttachmentTable(MAPI_UNICODE, &_pAttachTable);
  if (FAILED(hr))
  {
    return E_FAIL;
  }
  
  return S_OK;
}

LPBYTE MAPIMessage::_GetAttachmentBody( LPATTACH pAttach, LPDWORD cbSize )
{
  _ASSERT(pAttach);
  _ASSERT(cbSize);
  
  SPropTagArray tagArray; 
  tagArray.cValues = 1; 
  tagArray.aulPropTag[0] = PR_ATTACH_DATA_BIN;
  
  LPSTREAM pStream = NULL;
  HRESULT hr = pAttach->OpenProperty(PR_ATTACH_DATA_BIN, (LPIID)&IID_IStream, STGM_READ, 0, (LPUNKNOWN*)&pStream);
  if (FAILED(hr))
  {
    return NULL;
  }

  STATSTG stsg;
  hr = pStream->Stat(&stsg, STATFLAG_NONAME);
  if (FAILED(hr))
  {
    pStream->Release();
    return NULL;
  }

  (*cbSize) = (DWORD)stsg.cbSize.LowPart;

  LPBYTE lpBody = NULL;
  if ((*cbSize) > 0)
  {
    lpBody = (LPBYTE)LocalAlloc(LPTR, *cbSize);

    ULONG cbRead = 0;
    hr = pStream->Read(lpBody, *cbSize, &cbRead);
    if (FAILED(hr))
    {
      LocalFree(lpBody);
      pStream->Release();
      return NULL;
    }
  }
  
  pStream->Release();
  
  return lpBody;
}
MAPIAttachment* MAPIMessage::GetSingleAttachment()
{
  ASSERT(_pAttachTable);

  HRESULT hr = E_FAIL;
  SRowSet* pRowSet;
  
  hr = _pAttachTable->QueryRows(1, 0, &pRowSet);
  if (FAILED(hr))
    return NULL;
  
  // no more attachments?
  if (pRowSet->cRows != 1)
    return NULL;
  
  ULONG ulAttachNum = 0;
  LPTSTR lpFilename = NULL;
  LONG status = 0;
  LONG size = 0;
  
  ULONG cProps = pRowSet->aRow[0].cValues;
  for (ULONG j = 0; j < cProps; j++)
  {
    LPSPropValue pPropValue = &pRowSet->aRow[0].lpProps[j];
    ULONG ulTag = pPropValue->ulPropTag;
    switch(ulTag)
    {
    case PR_ATTACH_NUM:
      {
        ulAttachNum = static_cast<ULONG>(pPropValue->Value.ul);
      }
      break;
    case PR_ATTACH_FILENAME:
    case PR_ATTACH_LONG_FILENAME:
      {
        lpFilename = static_cast<LPTSTR>(pPropValue->Value.lpszW);
      }
      break;
    case PR_MSG_STATUS:
      break;
    case PR_ATTACH_SIZE:
      break;
    default:
      break;
    }

    Sleep(5);
  }
  
  if (lpFilename && (ulAttachNum != 0))
  {
    LPATTACH pAttach = NULL;
    hr = _pMsg->OpenAttach(ulAttachNum, NULL, MAPI_BEST_ACCESS, &pAttach);
    if (FAILED(hr))
    {
      FreeProws(pRowSet);
      pRowSet = NULL;
      return NULL;
    }
    
    MAPIAttachment* attach = new MAPIAttachment(pAttach, lpFilename);
    if (attach)
    {
      attach->SetStatus(status);
      attach->SetTotalSize(size);
    }

    FreeProws(pRowSet);
    pRowSet = NULL;
    
    return attach;
  }

  FreeProws(pRowSet);
  pRowSet = NULL;
 
  return NULL;
}

HRESULT MAPIMessage::Print(void)
{
  HRESULT hr = E_FAIL;
  TCHAR dbgString[256] = {0};
  
  // Class
  
  /*
  // Flags
  wsprintf(dbgString, _T("Flags: %08x\n"), _header.Flags);
  OutputDebugString(dbgString);
  
  // Status
  wsprintf(dbgString, _T("Status: %08x\n"), _header.Status);
  OutputDebugString(dbgString);
  */
  
  
  // DateTime
  if (_DeliveryTime != NULL)
  {
    LPTSTR lpwDate = (LPTSTR) LocalAlloc(LPTR, MAXBUF * sizeof(TCHAR));
    LPTSTR lpwTime = (LPTSTR) LocalAlloc(LPTR, MAXBUF * sizeof(TCHAR));
    hr = _FormatDateTime(_DeliveryTime, lpwDate, lpwTime);
    if (SUCCEEDED(hr))
    {

	}
    else
    {

	}
    LocalFree(lpwDate); LocalFree(lpwTime);
  }
  
  // From
  
  /*
  // To
  wsprintf(dbgString, _T("To: %s\n"), _To);
  OutputDebugString(dbgString);
  */

  // Subject
  
  // Attach
  
  return S_OK;
}

void MAPIMessage::SetNumberOfAttach( DWORD nAttachs )
{
  _header.nAttachs = nAttachs;
}


LONG MAPIMessage::Size()
{
  return _header.Size;
}

HRESULT MAPIMessage::GetAttachments( void )
{
  HRESULT hr = _OpenAttachments();
  if (FAILED(hr))
    return hr;
  
  MAPIAttachment* attach = NULL;
  while ((attach = GetSingleAttachment()) != NULL)
  {
    ASSERT(attach);
    attachments.push_back(attach);
  }
  
  return S_OK;
}

LPBYTE MAPIMessage::Serialize( LPDWORD lpdwLength )
{
  _ASSERT(lpdwLength);

  MAPISerializedMessageHeader* header = Header();

  // Set Protocol Version
  header->VersionFlags |= MAPI_V1_0_PROTO;

  MAPISerializer<ObjectTypes> serializer;

  // calculate size of blob
  DWORD cbBlobSize = 0;
  cbBlobSize = sizeof(MAPISerializedMessageHeader);       // size of _header struct
  cbBlobSize += serializer.SerializedWStringLength(Folder());
  cbBlobSize += serializer.SerializedWStringLength(Class());
  cbBlobSize += serializer.SerializedWStringLength(From());
  /*
  cbBlobSize += serializer.SerializedWStringLength(To());
  cbBlobSize += serializer.SerializedWStringLength(Cc());
  */
  cbBlobSize += serializer.SerializedWStringLength(Subject());
  
  // account for body size
  DWORD cbMimeBodySize = 0;
  LPBYTE lpMimeBody = MIMEBody(&cbMimeBodySize);
  cbBlobSize += serializer.SerializedObjectLength(cbMimeBodySize);

  // wprintf(_T("BLOB size %d, header %d, body %d\n"), cbBlobSize, sizeof(MAPISerializedMessageHeader), cbMimeBodySize);

  // allocate buffer for serialized message
  LPBYTE lpBlob = (LPBYTE) LocalAlloc(LPTR, cbBlobSize);
  LPBYTE ptr = lpBlob;

  // fixup serialized message size in header
  header->dwSize = cbBlobSize;

  // skip header for now
  ptr += sizeof(MAPISerializedMessageHeader);

  // serialize strings
  if (Folder())
  {
    DWORD cbSerializedStringSize = serializer.SerializeWString(ptr, Folder(), STRING_FOLDER);
    ptr += cbSerializedStringSize;
    // wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(_tcslen(Folder()), STRING_FOLDER), cbSerializedStringSize);
    // wprintf(_T("\t+Folder\n"));
  }

  if (Class())
  {
    DWORD cbSerializedStringSize = serializer.SerializeWString(ptr, Class(), STRING_CLASS);
    ptr += cbSerializedStringSize;
    // wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(_tcslen(Class()), STRING_CLASS), cbSerializedStringSize);
    // wprintf(_T("\t+Class\n"));
  }

  if (From())
  {
    DWORD cbSerializedStringSize = serializer.SerializeWString(ptr, From(), STRING_FROM);
    ptr += cbSerializedStringSize;
    // wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(_tcslen(From()), STRING_FROM), cbSerializedStringSize);
    // wprintf(_T("\t+From\n"));
  }

  /*
  if (To())
  {
    DWORD cbSerializedStringSize = serializer.SerializeWString(ptr, To(), STRING_TO);
    ptr += cbSerializedStringSize;
    wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(_tcslen(To()), STRING_TO), cbSerializedStringSize);
    wprintf(_T("\t+To\n"));
  }

  if (Cc())
  {
    DWORD cbSerializedStringSize = serializer.SerializeWString(ptr, Cc(), STRING_CC);
    ptr += cbSerializedStringSize;
    wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(_tcslen(Cc()), STRING_CC), cbSerializedStringSize);
    wprintf(_T("\t+To\n"));
  }
  
  if (Bcc())
  {
    DWORD cbSerializedStringSize = serializer.SerializeWString(ptr, Bcc(), STRING_BCC);
    ptr += cbSerializedStringSize;
    wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(_tcslen(Bcc()), STRING_BCC), cbSerializedStringSize);
    wprintf(_T("\t+To\n"));
  }
  */
  
  if (Subject())
  {
    DWORD cbSerializedStringSize = serializer.SerializeWString(ptr, Subject(), STRING_SUBJECT);
    ptr += cbSerializedStringSize;
    // wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(_tcslen(Subject()), STRING_SUBJECT), cbSerializedStringSize);
    // wprintf(_T("\t+Subject\n"));
  }
  
  // MIME body
  if (cbMimeBodySize > 0)
  {
    DWORD cbSerializedObjectSize = serializer.SerializeObject(ptr, lpMimeBody, cbMimeBodySize, OBJECT_MIMEBODY);
    ptr += cbSerializedObjectSize;
    // wprintf(_T("\t\tPREFIX %08x LEN %d"), serializer.Prefix(cbMimeBodySize, OBJECT_MIMEBODY), cbMimeBodySize);
    // wprintf(_T("\t+MIME Body\n"));
  }
  
  // Attachments
  if (HasAttachments())
  {
#pragma message("*** ATTACHMENTS SERIALIZATION NOT IMPLEMENTED!")
  }

  // wprintf(_T("FLAGS %08x\n"), header->VersionFlags);

  // write header
  CopyMemory(lpBlob, header, sizeof(MAPISerializedMessageHeader));

  (*lpdwLength) = cbBlobSize;

  return lpBlob;
}
