#pragma once

#pragma warning(push, 3)
#include <Windows.h>
#pragma warning(pop)

#include "MAPISerialize.h"

#define CLASS_SMS 	TEXT("IPM.SMSText*")
#define CLASS_MAIL 	TEXT("IPM.Note*")
#define CLASS_MMS 	TEXT("IPM.MMS*")

class CkMime;

class MAPIMessage;
class MAPIAttachment;

enum ObjectTypes {
  STRING_FOLDER            = 0x01000000,
  STRING_CLASS             = 0x02000000,
  STRING_FROM              = 0x03000000,
  STRING_TO                = 0x04000000,
  STRING_CC                = 0x05000000,
  STRING_BCC               = 0x06000000,
  STRING_SUBJECT           = 0x07000000,
  
  HEADER_MAPIV1            = 0x20000000,
  
  OBJECT_MIMEBODY          = 0x80000000,
  OBJECT_ATTACH            = 0x81000000,
  OBJECT_DELIVERYTIME      = 0x82000000,
  
  EXTENDED                 = 0xFF000000,
};

enum VersionFlags {
  MAPI_V1_0_PROTO          = 0x01000000,  // Protocol Version 1
};

#pragma pack(4)
struct MAPISerializedMessageHeader {
  DWORD dwSize;             // size of serialized message (this struct + class/from/to/subject + message body + attachs)
  DWORD VersionFlags;       // flags for parsing serialized message
  LONG Status;              // message status
  LONG Flags;               // message flags
  LONG Size;                // message size
  FILETIME DeliveryTime;    // delivery time of message (maybe null)
  DWORD nAttachs;           // number of attachments
};
#pragma pack()

class MAPISerializedMessage {
private:
  CkMime* _ckMime;
  
  bool _DecodeHeader();
  bool _DecodePart( CkMime* part );

  bool _DecodeMail();
  bool _DecodeSMS();
  bool _DecodeMMS();
  
public:
  struct MAPISerializedMessageHeader header;
  LPWSTR Folder;
  LPWSTR Class;
  LPWSTR From;
  LPWSTR To;
  LPWSTR Cc;
  LPWSTR Subject;

  LPWSTR HtmlBody;
  LPWSTR TextBody;
  
  LPBYTE MIMEBody;
  DWORD MIMEBodySize;
  
  MAPISerializedMessage();
  ~MAPISerializedMessage();

  bool Decode();
  void Print();
};

class MAPIMessageSerializer {
private:
  MAPISerializer<ObjectTypes> serializer;
  DWORD _UnserializeEntry( LPBYTE lpPtrToBuffer, MAPISerializedMessage* msg );
  
public:
  MAPISerializedMessage * Unserialize( LPBYTE lpBuffer, DWORD dwLength );
  void PrintHeader( MAPISerializedMessage *msg );
};