#include <map>
#include "PoomContactsReader.h"
#include "PoomContact.h"

#include <atlbase.h>

#define BUF_SIZE 11
#define MAX_DATE_DOUBLE_4000 767011.083333

CPoomContactsReader::CPoomContactsReader(IFolder* pIFolder)
	:	IPoomFolderReader(pIFolder)
{
	_items = IPoomFolderReader::getItemCollection();
}

CPoomContactsReader::~CPoomContactsReader(void)
{
	if(_items)
		_items->Release();
}

int CPoomContactsReader::Count()
{
	int uCount = 0;

	_items->get_Count(&uCount);

	return uCount;
}

HRESULT CPoomContactsReader::Get(int i, CPoomSerializer *pPoomSerializer, LPBYTE *pBuf, UINT *puBufLen)
{
	HRESULT hr = S_FALSE;

	if(_items == NULL)
		return hr;

	IContact *pElementContact = NULL;

	if(FAILED(_items->Item(i, (IDispatch **) &pElementContact)))
		return hr;

	CPoomContact *contact = new CPoomContact();
	HeaderStruct *header = contact->Header();
	
	if(SUCCEEDED(pElementContact->get_Oid(&header->lOid))){
		Parse(pElementContact, contact);
		*pBuf = pPoomSerializer->Serialize(contact, (LPDWORD) puBufLen);
	}

	if(contact)	delete contact;
	if(pElementContact) pElementContact->Release();

	return S_OK;
}

HRESULT CPoomContactsReader::GetOnce(IPOutlookApp *pIPoomApp, LONG lOid, CPoomSerializer *pPoomSerializer, LPBYTE *pBuf, UINT *puBufLen)
{
	HRESULT hr = S_FALSE;

	if(pIPoomApp == NULL)
		return hr;

	IContact *pElementContact = NULL;

	if(FAILED(pIPoomApp->GetItemFromOid(lOid, (IDispatch **) &pElementContact)))
		return hr;

	CPoomContact *calendar = new CPoomContact();
	HeaderStruct *header = calendar->Header();

	header->lOid = lOid;
	Parse(pElementContact, calendar);
	*pBuf = pPoomSerializer->Serialize(calendar, (LPDWORD) puBufLen);

	if(calendar) delete calendar;
	if(pElementContact)	pElementContact->Release();

	return S_OK;
}

void CPoomContactsReader::Parse(IContact *iContact, CPoomContact *contact)
{
	DATE date;
	SYSTEMTIME st;
	ZeroMemory(&date, sizeof(DATE));
	ZeroMemory(&st, sizeof(SYSTEMTIME));

	ContactMapType* contactMap = contact->Map();

	//CComBSTR test = NULL;	
/*
#define GET_ENTRY(func, e_type, pMap)\
	do{\
		if(SUCCEEDED(func(&test))){\
			if(test.Length() > 0){\
				LPWSTR lpwString = new WCHAR[test.Length() +1];\
				ZeroMemory(lpwString, sizeof(WCHAR)*(test.Length()));\
				StringCchCopy(lpwString, test.Length()+ 1, (LPWSTR) *(&test));\
				(*pMap)[e_type] = lpwString;\
			}\
			test.Empty();\
		}\
	}while(0)
*/
		BSTR bstrTemp;

#define GET_ENTRY(func, e_type, pMap)				\
	do {											\
		if(SUCCEEDED(func(&bstrTemp))){				\
			if(SysStringLen(bstrTemp) > 0){			\
				LPWSTR lpwString = new WCHAR[SysStringLen(bstrTemp) + 1];				\
				StringCchCopy(lpwString, SysStringLen(bstrTemp) + 1,(LPWSTR) bstrTemp);	\
				(*pMap)[e_type] = lpwString;			\
			}											\
		}												\
		SysFreeString(bstrTemp);						\
	} while (0)

	GET_ENTRY(iContact->get_FirstName, FirstName, contactMap);
	GET_ENTRY(iContact->get_LastName, LastName, contactMap);
	GET_ENTRY(iContact->get_CompanyName, CompanyName, contactMap);
	GET_ENTRY(iContact->get_BusinessFaxNumber, BusinessFaxNumber, contactMap);

	GET_ENTRY(iContact->get_Department, Department, contactMap);
	GET_ENTRY(iContact->get_Email1Address, Email1Address, contactMap);
	GET_ENTRY(iContact->get_MobileTelephoneNumber, MobileTelephoneNumber, contactMap);
	GET_ENTRY(iContact->get_OfficeLocation, OfficeLocation, contactMap);

	GET_ENTRY(iContact->get_PagerNumber, PagerNumber, contactMap);
	GET_ENTRY(iContact->get_BusinessTelephoneNumber, BusinessTelephoneNumber, contactMap);
	GET_ENTRY(iContact->get_JobTitle, JobTitle, contactMap);
	GET_ENTRY(iContact->get_HomeTelephoneNumber, HomeTelephoneNumber, contactMap);

	GET_ENTRY(iContact->get_Email2Address, Email2Address, contactMap);
	GET_ENTRY(iContact->get_Spouse, Spouse, contactMap);
	GET_ENTRY(iContact->get_Email3Address, Email3Address, contactMap);
	GET_ENTRY(iContact->get_Home2TelephoneNumber, Home2TelephoneNumber, contactMap);

	GET_ENTRY(iContact->get_HomeFaxNumber, HomeFaxNumber, contactMap);
	GET_ENTRY(iContact->get_CarTelephoneNumber, CarTelephoneNumber, contactMap);
	GET_ENTRY(iContact->get_AssistantName, AssistantName, contactMap);
	GET_ENTRY(iContact->get_AssistantTelephoneNumber, AssistantTelephoneNumber, contactMap);

	GET_ENTRY(iContact->get_Children, Children, contactMap);
	GET_ENTRY(iContact->get_Categories, Categories, contactMap);
	GET_ENTRY(iContact->get_WebPage, WebPage, contactMap);
	GET_ENTRY(iContact->get_Business2TelephoneNumber, Business2TelephoneNumber, contactMap);

	GET_ENTRY(iContact->get_RadioTelephoneNumber, RadioTelephoneNumber, contactMap);
	GET_ENTRY(iContact->get_FileAs, FileAs, contactMap);
	GET_ENTRY(iContact->get_YomiCompanyName, YomiCompanyName, contactMap);
	GET_ENTRY(iContact->get_YomiFirstName, YomiFirstName, contactMap);

	GET_ENTRY(iContact->get_YomiLastName, YomiLastName, contactMap);
	GET_ENTRY(iContact->get_Title, Title, contactMap);
	GET_ENTRY(iContact->get_MiddleName, MiddleName, contactMap);
	GET_ENTRY(iContact->get_Suffix, Suffix, contactMap);

	GET_ENTRY(iContact->get_HomeAddressStreet, HomeAddressStreet, contactMap);
	GET_ENTRY(iContact->get_HomeAddressCity, HomeAddressCity, contactMap);
	GET_ENTRY(iContact->get_HomeAddressState, HomeAddressState, contactMap);
	GET_ENTRY(iContact->get_HomeAddressPostalCode, HomeAddressPostalCode, contactMap);

	GET_ENTRY(iContact->get_HomeAddressCountry, HomeAddressCountry, contactMap);
	GET_ENTRY(iContact->get_OtherAddressStreet, OtherAddressStreet, contactMap);
	GET_ENTRY(iContact->get_OtherAddressCity, OtherAddressCity, contactMap);
	GET_ENTRY(iContact->get_OtherAddressPostalCode, OtherAddressPostalCode, contactMap);

	GET_ENTRY(iContact->get_OtherAddressCountry, OtherAddressCountry, contactMap);
	GET_ENTRY(iContact->get_BusinessAddressStreet, BusinessAddressStreet, contactMap);
	GET_ENTRY(iContact->get_BusinessAddressCity, BusinessAddressCity, contactMap);
	GET_ENTRY(iContact->get_BusinessAddressState, BusinessAddressState, contactMap);

	GET_ENTRY(iContact->get_BusinessAddressPostalCode, BusinessAddressPostalCode, contactMap);
	GET_ENTRY(iContact->get_BusinessAddressCountry, BusinessAddressCountry, contactMap);
	GET_ENTRY(iContact->get_CompanyName, Body, contactMap);
	GET_ENTRY(iContact->get_CompanyName, CompanyName, contactMap);

	GET_ENTRY(iContact->get_OtherAddressState, OtherAddressState, contactMap);
	GET_ENTRY(iContact->get_Body, Body, contactMap);

	if(SUCCEEDED(iContact->get_Birthday(&date))) {
		if(date != DATE_NONE && date < MAX_DATE_DOUBLE_4000){
			WCHAR *tmp1 = new WCHAR[BUF_SIZE];
			VariantTimeToSystemTime(date, &st);
			swprintf(tmp1, L"%d/%d/%d", st.wDay, st.wMonth, st.wYear);
			(*contactMap)[Birthday] = tmp1;
		}
	}

	if(SUCCEEDED(iContact->get_Anniversary(&date))) {
		if(date != DATE_NONE && date < MAX_DATE_DOUBLE_4000){
			WCHAR *tmp2 = new WCHAR[BUF_SIZE];
			VariantTimeToSystemTime(date, &st);
			swprintf(tmp2, L"%d/%d/%d", st.wDay, st.wMonth, st.wYear);
			(*contactMap)[Anniversary] = tmp2; 
		}
	}
}